import cython
cimport cython
import numpy as np
cimport numpy as np
import pandas as pd

asciifunc = np.frompyfunc(ascii, 1, 1)
reprfunc = np.frompyfunc(repr, 1, 1)
cdef:
    str ResetAll = "\033[0m"
    str Bold = "\033[1m"
    str Dim = "\033[2m"
    str Underlined = "\033[4m"
    str Blink = "\033[5m"
    str Reverse = "\033[7m"
    str Hidden = "\033[8m"
    str ResetBold = "\033[21m"
    str ResetDim = "\033[22m"
    str ResetUnderlined = "\033[24m"
    str ResetBlink = "\033[25m"
    str ResetReverse = "\033[27m"
    str ResetHidden = "\033[28m"
    str Default = "\033[39m"
    str Black = "\033[30m"
    str Red = "\033[31m"
    str Green = "\033[32m"
    str Yellow = "\033[33m"
    str Blue = "\033[34m"
    str Magenta = "\033[35m"
    str Cyan = "\033[36m"
    str LightGray = "\033[37m"
    str DarkGray = "\033[90m"
    str LightRed = "\033[91m"
    str LightGreen = "\033[92m"
    str LightYellow = "\033[93m"
    str LightBlue = "\033[94m"
    str LightMagenta = "\033[95m"
    str LightCyan = "\033[96m"
    str White = "\033[97m"
    str BackgroundDefault = "\033[49m"
    str BackgroundBlack = "\033[40m"
    str BackgroundRed = "\033[41m"
    str BackgroundGreen = "\033[42m"
    str BackgroundYellow = "\033[43m"
    str BackgroundBlue = "\033[44m"
    str BackgroundMagenta = "\033[45m"
    str BackgroundCyan = "\033[46m"
    str BackgroundLightGray = "\033[47m"
    str BackgroundDarkGray = "\033[100m"
    str BackgroundLightRed = "\033[101m"
    str BackgroundLightGreen = "\033[102m"
    str BackgroundLightYellow = "\033[103m"
    str BackgroundLightBlue = "\033[104m"
    str BackgroundLightMagenta = "\033[105m"
    str BackgroundLightCyan = "\033[106m"
    str BackgroundWhite = "\033[107m"



cpdef str printdf(
    object df,
    Py_ssize_t column_rep=70,
    Py_ssize_t max_lines=70,
    Py_ssize_t max_colwidth=300,
    Py_ssize_t ljust_space=2,
    str sep=" | ",
    bint vtm_escape=True,
):
    cdef:
        dict[Py_ssize_t, np.ndarray] stringdict
        dict[Py_ssize_t, Py_ssize_t] stringlendict
        list[str] df_columns, allcolumns_as_string, colors2rotate
        Py_ssize_t i, len_a, len_df_columns, lenstr, counter, j, len_stringdict0, k, len_stringdict
        str stringtoprint, dashes, dashesrep, string2print, string2printcolored
        np.ndarray a
    colors2rotate = [
        LightRed,
        LightGreen,
        LightYellow,
        LightBlue,
        LightMagenta,
        LightCyan,
        White,
    ]
    if vtm_escape:
        print('\033[12:2p')
    stringdict = {}
    if len(df) > max_lines and max_lines > 0:
        a = df.iloc[:max_lines].reset_index(drop=False).T.__array__()
    else:
        a = df.iloc[:len(df)].reset_index(drop=False).T.__array__()
    stringlendict = {}
    try:
        df_columns = ["iloc"] + [str(x) for x in df.columns]
    except Exception:
        try:
            df_columns = ["iloc",str(df.name)]
        except Exception:
            df_columns = ["iloc",str(0)]
    len_a=len(a)
    for i in range(len_a):
        try:
            #stringdict[i] = a[i].astype("U")
            stringdict[i] = reprfunc(a[i]).astype("U")
        except Exception:
            stringdict[i] = asciifunc(a[i]).astype("U")
        stringlendict[i] = (stringdict[i].dtype.itemsize // 4) + ljust_space
    for i in range(len_a):
        lenstr = len(df_columns[i])
        if lenstr > stringlendict[i]:
            stringlendict[i] = lenstr + ljust_space
        if max_colwidth > 0:
            if stringlendict[i] > max_colwidth:
                stringlendict[i] = max_colwidth

    allcolumns_as_string = []
    len_df_columns=len(df_columns)
    for i in range(len_df_columns):
        stringtoprint = str(df_columns[i])[: stringlendict[i]].ljust(stringlendict[i])
        allcolumns_as_string.append(stringtoprint)
    allcolumns_as_string_str = sep.join(allcolumns_as_string) + sep
    dashes = "-" * (len(allcolumns_as_string_str) + 2)
    dashesrep = dashes + "\n" + allcolumns_as_string_str + "\n" + dashes
    counter = 0
    len_stringdict0 = len(stringdict[0])
    len_stringdict=len(stringdict)
    for j in range(len_stringdict0):
        if column_rep > 0:
            if counter % column_rep == 0:
                print(dashesrep)
        counter += 1
        for k in range(len_stringdict):
            print((
                colors2rotate[k % len(colors2rotate)] + stringdict[k][j][: stringlendict[k]].replace("\n",
            "\\n").replace("\r", "\\r").ljust(stringlendict[k]) + ResetAll
            ), end=sep)
        print()
    return ""




import numpy as np
import os
import platform

numpyincludefolder = np.get_include()
name = "cythonprinter"

iswindows = platform.system() == "Windows"
# Explanation: https://cython.readthedocs.io/en/latest/src/userguide/source_files_and_compilation.html
# Most of the time, I use this configuration:
optionsdict = {
    "Options.docstrings": False,
    "Options.embed_pos_in_docstring": False,
    "Options.generate_cleanup_code": False,
    "Options.clear_to_none": True,
    "Options.annotate": True,
    "Options.fast_fail": False,
    "Options.warning_errors": False,
    "Options.error_on_unknown_names": True,
    "Options.error_on_uninitialized": True,
    "Options.convert_range": True,
    "Options.cache_builtins": True,
    "Options.gcc_branch_hints": True,
    "Options.lookup_module_cpdef": False,
    "Options.embed": False,
    "Options.cimport_from_pyx": False,
    "Options.buffer_max_dims": 8,
    "Options.closure_freelist_size": 8,
}

configdict = {
    "py_limited_api": False,
    "name": name,
    "sources": [name + ".pyx"],
    "include_dirs": [numpyincludefolder],
    "define_macros": [
        ("NPY_NO_DEPRECATED_API", 1),
        ("NPY_1_7_API_VERSION", 1),
        ("CYTHON_USE_DICT_VERSIONS", 1),
        ("CYTHON_FAST_GIL", 1),
        ("CYTHON_USE_PYLIST_INTERNALS", 1),
        ("CYTHON_USE_UNICODE_INTERNALS", 1),
        ("CYTHON_ASSUME_SAFE_MACROS", 1),
        ("CYTHON_USE_TYPE_SLOTS", 1),
        ("CYTHON_USE_PYTYPE_LOOKUP", 1),
        ("CYTHON_USE_ASYNC_SLOTS", 1),
        ("CYTHON_USE_PYLONG_INTERNALS", 1),
        ("CYTHON_USE_UNICODE_WRITER", 1),
        ("CYTHON_UNPACK_METHODS", 1),
        ("CYTHON_USE_EXC_INFO_STACK", 1),
        ("CYTHON_ATOMICS", 1),
    ],
    "undef_macros": [],
    "library_dirs": [],
    "libraries": [],
    "runtime_library_dirs": [],
    "extra_objects": [],
    "extra_compile_args": [],
    "extra_link_args": [],
    "export_symbols": [],
    "swig_opts": [],
    "depends": [],
    "language": "c",
    "optional": None,
}
compiler_directives = {
    "binding": True,
    "boundscheck": False,
    "wraparound": False,
    "initializedcheck": False,
    "nonecheck": False,
    "overflowcheck": False,
    "overflowcheck.fold": True,
    "embedsignature": False,
    "embedsignature.format": "c",  # (c / python / clinic)
    "cdivision": True,
    "cdivision_warnings": False,
    "cpow": True,
    "always_allow_keywords": False,
    "c_api_binop_methods": False,
    "profile": False,
    "linetrace": False,
    "infer_types": True,
    "language_level": 3,  # (2/3/3str)
    "c_string_type": "bytes",  # (bytes / str / unicode)
    "c_string_encoding": "default",  # (ascii, default, utf-8, etc.)
    "type_version_tag": False,
    "unraisable_tracebacks": True,
    "iterable_coroutine": True,
    "annotation_typing": True,
    "emit_code_comments": True,
    "cpp_locals": False,
    "legacy_implicit_noexcept": False,
    "optimize.use_switch": True,
    "optimize.unpack_method_calls": True,
    "warn.undeclared": False,  # (default False)
    "warn.unreachable": True,  # (default True)
    "warn.maybe_uninitialized": False,  # (default False)
    "warn.unused": False,  # (default False)
    "warn.unused_arg": False,  # (default False)
    "warn.unused_result": False,  # (default False)
    "warn.multiple_declarators": True,  # (default True)
    "show_performance_hints": True,  # (default True)
}

compdi = configdict
clidict = compiler_directives
from Cython.Compiler import Options
import re

for key, value in optionsdict.items():
    check1 = re.findall(r"^Options\.[a-z_]+$", str(key))
    check2 = re.findall(r"^(True|False|\d+)$", str(value))
    if check1 and check2:
        exec(f"{key} = {value}", globals())
from setuptools import Extension, setup
from Cython.Build import cythonize

ext_modules = Extension(**compdi)

setup(name=name, ext_modules=cythonize(ext_modules, compiler_directives=clidict))



from pandas.core.base import PandasObject
import pandas as pd
from pandas.core.frame import DataFrame, Series, Index

try:
    from .cythonprinter import printdf as pdp
except Exception:
    import subprocess, os, sys

    oldworkingdict = os.getcwd()
    this_folder = os.sep.join(__file__.split(os.sep)[:-1])
    os.chdir(this_folder)
    compilefile = os.path.join(this_folder, "cythonprintercompile.py")
    subprocess.run(
        [sys.executable, compilefile, "build_ext", "--inplace"],
        env=os.environ.copy(),
        shell=True,
    )
    os.chdir(oldworkingdict)
    from .cythonprinter import printdf as pdp


def print_col_width_len(df):
    try:
        pdp(
            pd.DataFrame(
                [df.shape[0], df.shape[1]], index=["rows", "columns"]
            ).T.rename(
                {0: "DataFrame"},
            ),
        )
    except Exception:
        pdp(
            pd.DataFrame([df.shape[0]], index=["rows"]).T.rename({0: "Series"}),
        )


def pandasprintcolor(self):
    pdp(pd.DataFrame(self.reset_index().__array__(), columns=['index']+[str(x) for x in self.columns],copy=False))
    print_col_width_len(self.__array__())

    return ""


def copy_func(f):
    # https://stackoverflow.com/a/67083317/15096247
    # Create a lambda that mimics f
    g = lambda *args: f(*args)
    # Add any properties of f
    t = list(filter(lambda prop: not ("__" in prop), dir(f)))
    i = 0
    while i < len(t):
        setattr(g, t[i], getattr(f, t[i]))
        i += 1
    return g


def pandasprintcolor_s(self):
    print("")
    try:
        pdp(pd.DataFrame(self.reset_index().__array__(), columns=['index',self.name],copy=False))
    except Exception:
        pdp(pd.DataFrame(self.__array__(),copy=False))
    print_col_width_len(self.__array__())

    return ""


def pandasindexcolor(self):
    pdp(pd.DataFrame(self.__array__()[: self.print_stop].reshape((-1, 1))))
    return ""


def reset_print_options():
    PandasObject.__str__ = copy_func(PandasObject.old__str__)
    PandasObject.__repr__ = copy_func(PandasObject.old__repr__)
    DataFrame.__repr__ = copy_func(DataFrame.old__repr__)
    DataFrame.__str__ = copy_func(DataFrame.old__str__)
    Series.__repr__ = copy_func(Series.old__repr__)
    Series.__str__ = copy_func(Series.old__str__)
    Index.__repr__ = copy_func(Index.old__repr__)
    Index.__str__ = copy_func(Index.old__str__)


def substitute_print_with_color_print(
    print_stop: int = 69, max_colwidth: int = 300, repeat_cols: int = 70
):
    if not hasattr(pd, "color_printer_active"):
        PandasObject.old__str__ = copy_func(PandasObject.__str__)
        PandasObject.old__repr__ = copy_func(PandasObject.__repr__)
        DataFrame.old__repr__ = copy_func(DataFrame.__repr__)
        DataFrame.old__str__ = copy_func(DataFrame.__str__)
        Series.old__repr__ = copy_func(Series.__repr__)
        Series.old__str__ = copy_func(Series.__str__)
        Index.old__repr__ = copy_func(Index.__repr__)
        Index.old__str__ = copy_func(Index.__str__)

    PandasObject.__str__ = lambda x: pandasprintcolor(x)
    PandasObject.__repr__ = lambda x: pandasprintcolor(x)
    PandasObject.print_stop = print_stop
    PandasObject.max_colwidth = max_colwidth
    PandasObject.repeat_cols = repeat_cols
    DataFrame.__repr__ = lambda x: pandasprintcolor(x)
    DataFrame.__str__ = lambda x: pandasprintcolor(x)
    DataFrame.print_stop = print_stop
    DataFrame.max_colwidth = max_colwidth
    DataFrame.repeat_cols = repeat_cols
    Series.__repr__ = lambda x: pandasprintcolor_s(x)
    Series.__str__ = lambda x: pandasprintcolor_s(x)
    Series.print_stop = print_stop
    Series.max_colwidth = max_colwidth
    Series.repeat_cols = repeat_cols
    Index.__repr__ = lambda x: pandasindexcolor(x)
    Index.__str__ = lambda x: pandasindexcolor(x)
    Index.print_stop = print_stop
    Index.max_colwidth = max_colwidth
    Index.repeat_cols = 10000000
    pd.color_printer_activate = substitute_print_with_color_print
    pd.color_printer_reset = reset_print_options
    pd.color_printer_active = True

#pdp(pd.DataFrame(self.reset_index().__array__(), columns=['index']+[str(x) for x in self.columns]))
# pdp(pd.DataFrame(self.reset_index().__array__(), columns=['index',self.name]))
def qq_ds_print_nolimit(self, **kwargs):
    try:
        pdp(
            pd.DataFrame(self.reset_index().__array__(), columns=['index']+[str(x) for x in self.columns],copy=False),
            max_lines=0,
            **kwargs,
        )
        print_col_width_len(self.__array__())
    except Exception:
        try:
            pdp(
                pd.DataFrame(self.reset_index().__array__(), columns=['index',self.name],copy=False),
                max_lines=0,
            )
        except Exception:
            pdp(
                pd.DataFrame(self.__array__(),copy=False),
                max_lines=0,
            )
        print_col_width_len(self.__array__())
    return ""


def qq_d_print_columns(self, **kwargs):
    pdp(
        pd.DataFrame(self.columns.__array__().reshape((-1, 1))),
        max_colwidth=0,
        max_lines=0,
        **kwargs,
    )
    return ""


def qq_ds_print_index(self, **kwargs):
    pdp(pd.DataFrame(self.index.__array__().reshape((-1, 1))),    max_lines=0, max_colwidth=0, **kwargs)
    return ""


def add_printer(overwrite_pandas_printer=False):

    PandasObject.ds_color_print_all = qq_ds_print_nolimit
    DataFrame.d_color_print_columns = qq_d_print_columns
    DataFrame.d_color_print_index = qq_ds_print_index
    if overwrite_pandas_printer:
        substitute_print_with_color_print()
